@@ -1,6 +1,7 @@
 import numpy as np
 import polars as pl
 import polars.selectors as cs
+import re
 
 from .datetime import get_timedelta_str, get_timestamp_column
 
@@ -9,6 +10,7 @@ INTEGER_REGEX = r"^[-+]?\d+$"
 FLOAT_REGEX = r"^[-+]?(?:\d*[.,])?\d+(?:[eE][-+]?\d+)?$"
 BOOLEAN_REGEX = r"^(true|false|1|0|yes|ja|no|nein|t|f|y|j|n|ok|nok)$"
 BOOLEAN_TRUE_REGEX = r"^(true|1|yes|ja|t|y|j|ok)$"
+# Make 8-digit pattern more restrictive to exclude obvious non-dates
 DATETIME_REGEX = (
     r"^("
     r"\d{4}-\d{2}-\d{2}"  # ISO: 2023-12-31
@@ -17,10 +19,11 @@ DATETIME_REGEX = (
     r"|"
     r"\d{2}\.\d{2}\.\d{4}"  # German: 31.12.2023
     r"|"
-    r"\d{8}"  # Compact: 20231231
+    # Only match 8-digit numbers that look like reasonable dates
+    r"(?:19|20)\d{6}"  # Years 1900-2099: 20231231
     r")"
     r"([ T]\d{2}:\d{2}(:\d{2}(\.\d{1,6})?)?)?"  # Optional time: 23:59[:59[.123456]]
-    r"([+-]\d{2}:?\d{2}|Z)?"  # Optional timezone: +01:00, -0500, Z
+    r"([+-]\d{2}:?\d{2}|Z|UTC)?"  # Optional timezone: +01:00, -0500, Z, UTC
     r"$"
 )
 
@@ -106,68 +109,248 @@ def _optimize_numeric_column(
     return expr.shrink_dtype()
 
 
+def _detect_timezone_from_sample(series: pl.Series, sample_size: int = 100) -> str | None:
+    """Efficiently detect the most common timezone from a sample of datetime strings.
+
+    Args:
+        series: Series containing datetime strings
+        sample_size: Maximum number of values to sample (for performance)
+
+    Returns:
+        The most common timezone string, or None if most values are timezone-naive
+    """
+    # Sample the data for performance
+    sample_size = min(sample_size, len(series))
+    if sample_size == 0:
+        return None
+
+    # Take a sample from the series
+    if len(series) <= sample_size:
+        sample = series
+    else:
+        # Take every nth item to get a representative sample
+        step = len(series) // sample_size
+        sample = series[::step][:sample_size]
+
+    # Drop null values
+    sample = sample.drop_nulls()
+    if sample.is_empty():
+        return None
+
+    # Extract timezone information
+    timezones = []
+    utc_count = 0
+    naive_count = 0
+
+    for val in sample:
+        val_str = str(val)
+
+        # Check for timezone patterns
+        if val_str.endswith("Z"):
+            timezones.append("UTC")
+            utc_count += 1
+        elif "+00:00" in val_str:
+            timezones.append("UTC")
+            utc_count += 1
+        elif "+0000" in val_str:
+            timezones.append("UTC")
+            utc_count += 1
+        elif re.search(r"[+-]\d{2}:?\d{2}$", val_str):
+            # Extract timezone offset
+            match = re.search(r"([+-]\d{2}:?\d{2})$", val_str)
+            if match:
+                tz = match.group(1)
+                if tz == "+00:00" or tz == "+0000":
+                    timezones.append("UTC")
+                    utc_count += 1
+                else:
+                    timezones.append(tz)
+        else:
+            naive_count += 1
+
+    # Determine the most common timezone
+    if not timezones:
+        # All values are timezone-naive
+        return None
+
+    # Count timezone occurrences
+    from collections import Counter
+    tz_counts = Counter(timezones)
+
+    # If most values are timezone-naive, return None
+    if naive_count > len(sample) / 2:
+        return None
+
+    # Get the most common timezone
+    most_common_tz, most_common_count = tz_counts.most_common(1)[0]
+
+    # If there's a tie, prefer UTC
+    if most_common_count == utc_count and utc_count > 0:
+        return "UTC"
+
+    # If the most common timezone appears in more than 30% of non-naive values, use it
+    if most_common_count >= len(timezones) * 0.3:
+        return most_common_tz
+
+    # Default to UTC if we have any timezone information
+    return "UTC"
+
+
 def _optimize_string_column(
     series: pl.Series,
     shrink_numerics: bool,
     time_zone: str | None = None,
     allow_null: bool = True,
     allow_unsigned: bool = True,
+    force_timezone: str | None = None,
 ) -> pl.Expr:
-    """Convert string column to appropriate type based on content analysis."""
-    # Return early for empty or null-only series
+    """Convert string column to appropriate type based on content analysis.
+
+    Änderung:
+    Platzhalter-/Null-ähnliche Werte ("", "-", "None", "NaN", "null", usw.)
+    blockieren nicht mehr die Erkennung (Int/Float/Bool/Datetime). Für die
+    Musterprüfung werden sie herausgefiltert. Bei erfolgreichem Cast werden sie
+    durch das Cleaning (`_clean_string_expr`) zu Null. Fallback belässt die
+    Originalwerte (kein erzwungenes Nullen, wie angefragt).
+    """
     col_name = series.name
     cleaned_expr = _clean_string_expr(col_name)
-    non_null = series.drop_nulls()
-    if non_null.is_empty():
+
+    # Check if all values are actually null (including null-like strings)
+    cleaned_series = series.to_frame().select(_clean_string_expr(col_name)).to_series()
+    if cleaned_series.is_null().all():
+        if allow_null:
+            # Return a column of nulls with Null type
+            return pl.lit(None, dtype=pl.Null()).alias(col_name)
+        return pl.col(col_name)
+
+    stripped = series.drop_nulls().str.strip_chars()
+
+    # Liste der Platzhalter-Werte muss mit _clean_string_expr konsistent bleiben
+    null_like_values = [
+        "-",
+        "",
+        "None",
+        "none",
+        "NONE",
+        "NaN",
+        "Nan",
+        "nan",
+        "NAN",
+        "N/A",
+        "n/a",
+        "null",
+        "Null",
+        "NULL",
+    ]
+
+    detector_values = stripped.filter(~stripped.is_in(null_like_values))
+
+    # Wenn nach Herausfiltern nichts mehr übrig bleibt → Spalte nur Platzhalter
+    if detector_values.is_empty():
         if allow_null:
             return pl.col(col_name).cast(pl.Null())
-        else:
-            return pl.col(col_name).cast(series.dtype)
+        return pl.col(col_name)
 
-    stripped = non_null.str.strip_chars()
-    lowercase = stripped.str.to_lowercase()
+    lower_detector = detector_values.str.to_lowercase()
 
-    # Check for boolean values
-    if lowercase.str.contains(BOOLEAN_REGEX).all(ignore_nulls=False):
+    # Integer-Erkennung (vor Datetime, um 8-stellige Zahlen zu erfassen)
+    if detector_values.str.contains(INTEGER_REGEX).all():
+        # Cast to Int64 first to ensure valid conversion
+        int_expr = cleaned_expr.cast(pl.Int64)
+
+        # Check if we should use unsigned integers
+        if allow_unsigned:
+            # Check if all original string values are non-negative
+            # (excluding null-like values that were cleaned)
+            if detector_values.min() is not None and detector_values.min() >= "0":
+                # Convert to UInt64 first, then shrink if needed
+                uint_expr = int_expr.cast(pl.UInt64)
+                if shrink_numerics:
+                    return uint_expr.shrink_dtype().alias(col_name)
+                return uint_expr.alias(col_name)
+
+        # Fall back to signed integers
+        if shrink_numerics:
+            return int_expr.shrink_dtype().alias(col_name)
+        return int_expr.alias(col_name)
+
+    # Float-Erkennung
+    if detector_values.str.contains(FLOAT_REGEX).all():
+        float_expr = cleaned_expr.str.replace_all(",", ".").cast(pl.Float64).alias(
+            col_name
+        )
+        if shrink_numerics:
+            temp_floats = detector_values.str.replace_all(",", ".").cast(
+                pl.Float64, strict=False
+            )
+            if _can_downcast_to_float32(temp_floats):
+                return float_expr.shrink_dtype().alias(col_name)
+        return float_expr
+
+    # Boolean-Erkennung
+    if lower_detector.str.contains(BOOLEAN_REGEX).all():
         return (
             cleaned_expr.str.to_lowercase()
             .str.contains(BOOLEAN_TRUE_REGEX)
             .alias(col_name)
         )
 
-    elif stripped.str.contains(INTEGER_REGEX).all(ignore_nulls=False):
-        int_expr = cleaned_expr.cast(pl.Int64).alias(col_name)
-        return (
-            int_expr.shrink_dtype().alias(col_name)
-            if shrink_numerics
-            else int_expr.alias(col_name)
-        )
+    # Datetime-Erkennung mit Polars' eingebauter Format-Erkennung
+    if detector_values.str.contains(DATETIME_REGEX).all():
+        try:
+            # Prüfe ob gemischte Zeitzonen vorhanden sind
+            has_tz = series.str.contains(r"(Z|UTC|[+-]\d{2}:\d{2}|[+-]\d{4})$").any()
 
-    # Check for numeric values
-    elif stripped.str.contains(FLOAT_REGEX).all(ignore_nulls=False):
-        float_expr = (
-            cleaned_expr.str.replace_all(",", ".").cast(pl.Float64).alias(col_name)
-        )
+            if has_tz:
+                # Bei gemischten Zeitzonen, verwende eager parsing auf Series-Ebene
+                import re
 
-        if shrink_numerics:
-            # Check if values can fit in Float32
-            temp_floats = stripped.str.replace_all(",", ".").cast(
-                pl.Float64, strict=False
-            )
-            if _can_downcast_to_float32(temp_floats):
-                return float_expr.shrink_dtype().alias(col_name)
+                def normalize_datetime_string(s):
+                    """Normalisiere verschiedene Zeitzone-Formate für das Parsen."""
+                    if not s:
+                        return s
 
-        return float_expr
+                    s = str(s).strip()
+                    # Entferne Zeitzonen-Informationen, da Polars diese nicht gemischt verarbeiten kann
+                    s = re.sub(r'Z$', '', s)
+                    s = re.sub(r'UTC$', '', s)
+                    s = re.sub(r'([+-]\d{2}:\d{2})$', '', s)
+                    s = re.sub(r'([+-]\d{4})$', '', s)
+                    return s
 
-    try:
-        if stripped.str.contains(DATETIME_REGEX).all(ignore_nulls=False):
-            return cleaned_expr.str.to_datetime(
-                strict=False, time_unit="us", time_zone=time_zone
-            ).alias(col_name)
-    except pl.exceptions.PolarsError:
-        pass
+                # Normalisiere die Zeitzone-Formate
+                normalized_series = series.map_elements(normalize_datetime_string, return_dtype=pl.String)
 
-    # Keep original if no conversion applies
+                # Parse mit force_timezone falls angegeben
+                if force_timezone is not None:
+                    dt_series = normalized_series.str.to_datetime(time_zone=force_timezone, time_unit="us")
+                else:
+                    # Erkenne die häufigste Zeitzone
+                    detected_tz = _detect_timezone_from_sample(series)
+                    if detected_tz is not None:
+                        dt_series = normalized_series.str.to_datetime(time_zone=detected_tz, time_unit="us")
+                    else:
+                        dt_series = normalized_series.str.to_datetime(time_unit="us")
+
+                # Konvertiere zurück zu Expr
+                return pl.lit(dt_series).alias(col_name)
+            else:
+                # Keine gemischten Zeitzonen - normales expression-basiertes Parsen
+                dt_expr = cleaned_expr.str.to_datetime(strict=False, time_unit="us")
+
+                # Wende force_timezone an falls angegeben
+                if force_timezone is not None:
+                    dt_expr = dt_expr.dt.replace_time_zone(force_timezone)
+
+                return dt_expr.alias(col_name)
+        except Exception as e:  # pragma: no cover - defensive
+            # Log error for debugging
+            import logging
+            logging.debug(f"Datetime parsing failed for column {col_name}: {e}")
+            pass
+
+    # Kein Cast → Original behalten
     return pl.col(col_name)
 
 
@@ -178,6 +361,7 @@ def _get_column_expr(
     allow_unsigned: bool = True,
     allow_null: bool = True,
     time_zone: str | None = None,
+    force_timezone: str | None = None,
 ) -> pl.Expr:
     """Generate optimization expression for a single column."""
     series = df[col_name]
@@ -194,8 +378,14 @@ def _get_column_expr(
             series, shrink_numerics, allow_unsigned, allow_null
         )
     elif series.dtype == pl.Utf8:
+        # korrigierter Aufruf: (series, shrink_numerics, time_zone, allow_null, allow_unsigned)
         return _optimize_string_column(
-            series, col_name, shrink_numerics, time_zone, allow_null
+            series,
+            shrink_numerics,
+            time_zone,
+            allow_null,
+            allow_unsigned,
+            force_timezone,
         )
 
     # Keep original for other types
@@ -211,6 +401,8 @@ def opt_dtype(
     allow_unsigned: bool = True,
     allow_null: bool = True,
     strict: bool = False,
+    *,
+    force_timezone: str | None = None,
 ) -> pl.DataFrame:
     """
     Optimize data types of a Polars DataFrame for performance and memory efficiency.
@@ -220,17 +412,18 @@ def opt_dtype(
     numeric type downcasting.
 
     Args:
-        df: DataFrame to optimize
-        include: Column(s) to include in optimization (default: all columns)
-        exclude: Column(s) to exclude from optimization
-        time_zone: Optional time zone for datetime parsing
-        shrink_numerics: Whether to downcast numeric types when possible
-        allow_unsigned: Whether to allow unsigned integer types
-        allow_null: Whether to allow columns with all null values to be cast to Null type
-        strict: If True, will raise an error if any column cannot be optimized
+        df: The Polars DataFrame to optimize.
+        include: Column(s) to include in optimization (default: all columns).
+        exclude: Column(s) to exclude from optimization.
+        time_zone: Optional time zone hint during datetime parsing.
+        shrink_numerics: Whether to downcast numeric types when possible.
+        allow_unsigned: Whether to allow unsigned integer types.
+        allow_null: Whether to allow columns with all null values to be cast to Null type.
+        strict: If True, will raise an error if any column cannot be optimized.
+        force_timezone: If set, ensure all parsed datetime columns end up with this timezone.
 
     Returns:
-        DataFrame with optimized data types
+        DataFrame with optimized data types.
     """
     # Normalize include/exclude parameters
     if isinstance(include, str):
@@ -251,7 +444,13 @@ def opt_dtype(
         try:
             expressions.append(
                 _get_column_expr(
-                    df, col_name, shrink_numerics, allow_unsigned, allow_null, time_zone
+                    df,
+                    col_name,
+                    shrink_numerics,
+                    allow_unsigned,
+                    allow_null,
+                    time_zone,
+                    force_timezone,
                 )
             )
         except Exception as e:
@@ -264,165 +463,6 @@ def opt_dtype(
     return df if not expressions else df.with_columns(expressions)
 
 
-# def opt_dtype(
-#     df: pl.DataFrame,
-#     include: str | list[str] | None = None,
-#     exclude: str | list[str] | None = None,
-#     time_zone: str | None = None,
-#     shrink_numerics: bool = True,
-# ) -> pl.DataFrame:
-#     """
-#     Analyzes and optimizes the data types of a Polars DataFrame for performance
-#     and memory efficiency.
-
-#     This version includes:
-#     - Robust numeric, boolean, and datetime casting from strings.
-#     - Handling of whitespace and common null-like string values.
-#     - Casting of columns containing only nulls to pl.Int8.
-#     - Optional shrinking of numeric columns to the smallest possible type.
-
-#     Args:
-#         df: The DataFrame to optimize.
-#         include: A list of columns to forcefully include in the optimization.
-#         exclude: A list of columns to exclude from the optimization.
-#         time_zone: Optional time zone for datetime parsing.
-#         shrink_numerics: If True, numeric columns (both existing and newly converted from strings)
-#             will be downcast to the smallest possible type that can hold their values (e.g., Int64 to Int32, Float64 to Float32),
-#             similar to Polars' shrink_dtype() behavior. If False, this shrinking step is skipped.
-
-#     Returns:
-#         An optimized Polars DataFrame with improved data types.
-#     """
-#     # Phase 1: Analysis - Determine columns to process and build a list of
-#     # transformation expressions without executing them immediately.
-#     if isinstance(include, str):
-#         include = [include]
-#     if isinstance(exclude, str):
-#         exclude = [exclude]
-
-#     cols_to_process = df.columns
-#     if include:
-#         cols_to_process = [col for col in include if col in df.columns]
-#     if exclude:
-#         cols_to_process = [col for col in cols_to_process if col not in exclude]
-
-#     expressions = []
-#     for col_name in cols_to_process:
-#         s = df[col_name]
-
-#         # NEW: If a column is entirely null, cast it to Int8 and skip other checks.
-#         if s.is_null().all():
-#             expressions.append(pl.col(col_name).cast(pl.Int8))
-#             continue
-
-#         dtype = s.dtype
-
-#         # 1. Optimize numeric columns by shrinking their size
-#         if dtype.is_numeric():
-#             if shrink_numerics:
-#                 if dtype == pl.Float64:
-#                     column_series = df[col_name]
-#                     finite_values_series = column_series.filter(
-#                         column_series.is_finite()
-#                     )
-#                     can_shrink = True
-#                     if not finite_values_series.is_empty():
-#                         min_finite_val = finite_values_series.min()
-#                         max_finite_val = finite_values_series.max()
-#                         if (min_finite_val < F32_MIN_FINITE) or (
-#                             max_finite_val > F32_MAX_FINITE
-#                         ):
-#                             can_shrink = False
-#                     if can_shrink:
-#                         expressions.append(pl.col(col_name).shrink_dtype())
-#                     else:
-#                         expressions.append(pl.col(col_name))
-#                 else:
-#                     expressions.append(pl.col(col_name).shrink_dtype())
-#             else:
-#                 expressions.append(pl.col(col_name))
-#             continue
-
-#         # 2. Optimize string columns by casting to more specific types
-#         if dtype == pl.Utf8:
-#             # Create a cleaned column expression that first strips whitespace, then
-#             # replaces common null-like strings.
-#             cleaned_col = (
-#                 pl.col(col_name)
-#                 .str.strip_chars()
-#                 .replace({"-": None, "": None, "None": None})
-#             )
-
-#             # Analyze a stripped, non-null version of the series to decide the cast type
-#             s_non_null = s.drop_nulls()
-#             if len(s_non_null) == 0:
-#                 # The column only contains nulls or null-like strings.
-#                 # Cast to Int8 as requested for all-null columns.
-#                 expressions.append(pl.col(col_name).cast(pl.Int8))
-#                 continue
-
-#             s_stripped_non_null = s_non_null.str.strip_chars()
-
-#             # Check for boolean type
-#             if s_stripped_non_null.str.to_lowercase().str.contains(BOOLEAN_REGEX).all():
-#                 expr = cleaned_col.str.to_lowercase().str.contains(BOOLEAN_TRUE_REGEX)
-#                 expressions.append(expr.alias(col_name))
-#                 continue
-
-#             # Check for numeric type
-#             if s_stripped_non_null.str.contains(NUMERIC_REGEX).all():
-#                 is_float = s_stripped_non_null.str.contains(r"[.,eE]").any()
-#                 numeric_col = cleaned_col.str.replace_all(",", ".")
-#                 if is_float:
-#                     if shrink_numerics:
-#                         temp_float_series = s_stripped_non_null.str.replace_all(
-#                             ",", "."
-#                         ).cast(pl.Float64, strict=False)
-#                         finite_values_series = temp_float_series.filter(
-#                             temp_float_series.is_finite()
-#                         )
-#                         can_shrink = True
-#                         if not finite_values_series.is_empty():
-#                             min_finite_val = finite_values_series.min()
-#                             max_finite_val = finite_values_series.max()
-#                             if (min_finite_val < F32_MIN_FINITE) or (
-#                                 max_finite_val > F32_MAX_FINITE
-#                             ):
-#                                 can_shrink = False
-#                         base_expr = numeric_col.cast(pl.Float64)
-#                         if can_shrink:
-#                             expressions.append(base_expr.shrink_dtype().alias(col_name))
-#                         else:
-#                             expressions.append(base_expr.alias(col_name))
-#                     else:
-#                         expressions.append(numeric_col.cast(pl.Float64).alias(col_name))
-#                 else:
-#                     if shrink_numerics:
-#                         expressions.append(
-#                             numeric_col.cast(pl.Int64).shrink_dtype().alias(col_name)
-#                         )
-#                     else:
-#                         expressions.append(numeric_col.cast(pl.Int64).alias(col_name))
-#                 continue
-
-#             # Check for datetime type using a fast heuristic
-#             try:
-#                 if s_stripped_non_null.str.contains(DATETIME_REGEX).all():
-#                     expressions.append(
-#                         cleaned_col.str.to_datetime(
-#                             strict=False, time_unit="us", time_zone=time_zone
-#                         ).alias(col_name)
-#                     )
-#                     continue
-#             except pl.exceptions.PolarsError:
-#                 pass
-
-#     # Phase 2: Execution - If any optimizations were identified, apply them
-#     # all at once for maximum parallelism and performance.
-#     if not expressions:
-#         return df
-
-#     return df.with_columns(expressions)
 
 
 def unnest_all(df: pl.DataFrame, seperator="_", fields: list[str] | None = None):
@@ -772,7 +812,7 @@ def partition_by(
             timestamp_column=timestamp_column, strftime=strftime
         )
         strftime_columns = [
-            f"_strftime_{strftime_.replaace('%', '')}_" for strftime_ in strftime
+            f"_strftime_{strftime_.replace('%', '')}_" for strftime_ in strftime
         ]
         columns_ += strftime_columns
         drop_columns += strftime_columns
